'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _jsonSchemaRefParser = require('json-schema-ref-parser');

var _jsonSchemaRefParser2 = _interopRequireDefault(_jsonSchemaRefParser);

var _isNil = require('inspected/schema/is-nil');

var _isNil2 = _interopRequireDefault(_isNil);

var _isString = require('inspected/schema/is-string');

var _isString2 = _interopRequireDefault(_isString);

var _isObject = require('inspected/schema/is-object');

var _isObject2 = _interopRequireDefault(_isObject);

var _deepmerge = require('deepmerge');

var _deepmerge2 = _interopRequireDefault(_deepmerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const resolve = ({ dereference, parser, resolver }) => (() => {
  var _ref = _asyncToGenerator(function* (schema) {
    const refParserOptions = {
      parse: { parser },
      resolve: { custom: resolver }
    };

    return dereference ? yield _jsonSchemaRefParser2.default.dereference(schema, refParserOptions) : yield _jsonSchemaRefParser2.default.bundle(schema, refParserOptions);
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

const parse = options => (() => {
  var _ref2 = _asyncToGenerator(function* (schema) {
    if ((0, _isNil2.default)(schema)) {
      throw new Error('No schema specified.');
    }

    if (!(0, _isString2.default)(schema) && !(0, _isObject2.default)(schema)) {
      throw new Error('Schema must be a string path or spec object.');
    }

    // TODO: validate options against schema
    if (!(0, _isNil2.default)(options) && !(0, _isObject2.default)(options)) {
      throw new Error('Options should be an object.');
    }

    try {
      const defaultOptions = {
        dereference: false,
        parser: {
          canParse: function (file) {
            return false;
          },
          parse: (() => {
            var _ref3 = _asyncToGenerator(function* (file) {});

            return function parse(_x3) {
              return _ref3.apply(this, arguments);
            };
          })()
        },
        resolver: {
          canResolve: function (info) {
            return false;
          },
          resolve: (() => {
            var _ref4 = _asyncToGenerator(function* (info) {});

            return function resolve(_x4) {
              return _ref4.apply(this, arguments);
            };
          })()
        }
      };

      const compiledOptions = options ? (0, _deepmerge2.default)(defaultOptions, options) : defaultOptions;

      const parser = {
        order: 1,
        canParse: function (info) {
          return compiledOptions.parser.canParse(info);
        },
        parse: (() => {
          var _ref5 = _asyncToGenerator(function* (info) {
            return compiledOptions.parser.parse(info);
          });

          return function parse(_x5) {
            return _ref5.apply(this, arguments);
          };
        })()
      };

      const resolver = {
        order: 1,
        canRead: function (info) {
          return compiledOptions.resolver.canResolve({
            path: info.url,
            extension: info.extension
          });
        },
        read: (() => {
          var _ref6 = _asyncToGenerator(function* (info) {
            return yield compiledOptions.resolver.resolve({
              path: info.url,
              extension: info.extension
            });
          });

          return function read(_x6) {
            return _ref6.apply(this, arguments);
          };
        })()
      };

      return yield resolve({
        dereference: compiledOptions.dereference,
        parser,
        resolver
      })(schema);
    } catch (error) {
      throw new Error(`There was an error parsing the specified spec:\n${error}`);
    }
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();

exports.default = parse;